import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.ErrorHandler;
import org.xml.sax.Locator;
import org.xml.sax.SAXParseException;
import org.xml.sax.XMLReader;

public class DocumentPrinter implements ContentHandler, ErrorHandler
{
    private static final String PARSER_NAME 
        = "org.apache.xerces.parsers.SAXParser";
    

    public DocumentPrinter()
    {}

    public static void main(String[] args)
    {
        if (args.length == 0)
        {
            System.out.println("No XML document path specified.");
            System.exit(1);
        }

        DocumentPrinter dp = new DocumentPrinter();

        XMLReader parser;

        try
        {
            parser = (XMLReader)Class.forName(PARSER_NAME).newInstance();
            parser.setContentHandler(dp);
            parser.setErrorHandler(dp);
            parser.parse(args[0]);
        }
        // Normally it's a bad idea to catch generic exceptions like this.
        catch (Exception ex)
        {
            System.out.println(ex.getMessage());
            ex.printStackTrace();
        }
    }

    public void characters(char[] ch, int start, int length)
    {
        String chars = "";
        for (int i = start; i < start + length; i++)
        {
            chars = chars + ch[i];
        }
        
        System.out.println("Received characters: " + chars);
    }

    public void endDocument()
    {
        System.out.println("End of document reached.");
    }

    public void endElement(String namespaceURI, String localName,
                           String qName)
    {
        System.out.println("End of element: " + localName);
    }

    public void endPrefixMapping(String prefix)
    {
        System.out.println("End of prefix mapping: " + prefix);
    }

    public void ignorableWhitespace(char[] ch, int start, int length)
    {
        System.out.println("Received whitespace.");
    }

    public void processingInstruction(String target, String data)
    {
        System.out.println("Received processing instruction:");
        System.out.println("Target: " + target);
        System.out.println("Data: " + data);
    }

    public void setDocumentLocator(Locator locator)
    {
    }

    public void skippedEntity(String name)
    {
        System.out.println("Skipped entity: " + name);
    }

    public void startDocument() 
    {
        System.out.println("Start document.");
    }
    
    public void startElement(String namespaceURI, String localName, 
                             String qName, Attributes atts)
    {
        System.out.println("Start element: " + localName);
    }

    public void startPrefixMapping(String prefix, String uri)
    {
        System.out.println("Prefix mapping: " + prefix);
        System.out.println("URI: " + uri);
    }

    public void error(SAXParseException exception)
    {
        
    }

    public void fatalError(SAXParseException exception)
    {}

    public void warning(SAXParseException exception)
    {}
}
